﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using Microsoft.Xrm.Sdk;
using RoleUpdater.DelegatesHelpers;

namespace RoleUpdater.Controls
{
    public partial class RoleSelectionControl : UserControl
    {
        #region Variables

        /// <summary>
        /// List of existing roles
        /// </summary>
        private readonly List<Entity> _roles;

        private readonly UpdateSettings _settings;

        #endregion Variables

        #region Constructor

        /// <summary>
        /// Initializes a new instance of class RoleSelectionControl
        /// </summary>
        /// <param name="roles">List of existing roles</param>
        /// <param name="settings"> </param>
        public RoleSelectionControl(List<Entity> roles, UpdateSettings settings)
        {
            InitializeComponent();

            _roles = roles;
            _settings = settings;
        }

        #endregion Constructor

        #region Properties

        /// <summary>
        /// Gets the list of selected roles
        /// </summary>
        public List<Entity> SelectedRoles
        {
            get
            {
                return (from ListViewItem item in lvRoles.SelectedItems select (Entity) item.Tag).ToList();
            }
        }

        #endregion Properties

        #region Methods

        private void RoleSelectionControlLoad(object sender, EventArgs e)
        {
            // Loads roles in the list view control
            foreach (var role in _roles)
            {
                var item = new ListViewItem
                {
                    Text = role["name"].ToString(),
                    Tag = role
                };

                item.SubItems.Add(((EntityReference)role["businessunitid"]).Name);

                ListViewDelegates.AddItem(lvRoles, item);

                if(_settings != null && _settings.SelectedRoles.Count > 0)
                {
                    if (_settings.SelectedRoles.Any(x => x["name"] == role["name"]))
                        item.Selected = true;
                }
            }

            lvRoles.Columns[lvRoles.Columns.Count - 1].Width = -2;
        }

        #endregion Methods
    }
}
